@@ -5,6 +5,7 @@
  */
 
 #include "config.h"
+#include <glib/gi18n.h>
 
 #include "adw-toast-overlay.h"
 
@@ -62,7 +63,7 @@
  *
  * ## Accessibility
  *
- * `AdwToastOverlay` uses the `GTK_ACCESSIBLE_ROLE_TAB_GROUP` role.
+ * `AdwToastOverlay` uses the [enum@Gtk.AccessibleRole.GROUP] role.
  */
 
 typedef struct {
@@ -240,6 +241,8 @@ show_toast (AdwToastOverlay *self,
             ToastInfo       *info)
 {
   AdwAnimationTarget *target;
+  const char *title, *button_label;
+  char *announcement;
 
   g_assert (!info->widget);
 
@@ -260,6 +263,27 @@ show_toast (AdwToastOverlay *self,
                                              G_CALLBACK (show_done_cb), info);
 
   adw_animation_play (info->show_animation);
+
+  title = adw_toast_get_title (info->toast);
+  button_label = adw_toast_get_button_label (info->toast);
+
+  if (title && button_label) {
+    announcement = g_strdup_printf (_("A toast appeared: %s, has a button: %s"),
+                                    title, button_label);
+  } else if (title && !button_label) {
+    announcement = g_strdup_printf (_("A toast appeared: %s"), title);
+  } else if (!title && button_label) {
+    announcement = g_strdup_printf (_("A toast appeared, has a button: %s"),
+                                    button_label);
+  } else {
+    announcement = g_strdup (_("A toast appeared"));
+  }
+
+  gtk_accessible_announce (GTK_ACCESSIBLE (self),
+                           announcement,
+                           GTK_ACCESSIBLE_ANNOUNCEMENT_PRIORITY_MEDIUM);
+
+  g_free (announcement);
 }
 
 static int
@@ -498,7 +522,7 @@ adw_toast_overlay_class_init (AdwToastOverlayClass *klass)
   widget_class->size_allocate = adw_toast_overlay_size_allocate;
 
   /**
-   * AdwToastOverlay:child: (attributes org.gtk.Property.get=adw_toast_overlay_get_child org.gtk.Property.set=adw_toast_overlay_set_child)
+   * AdwToastOverlay:child:
    *
    * The child widget.
    */
@@ -561,7 +585,7 @@ adw_toast_overlay_new (void)
 }
 
 /**
- * adw_toast_overlay_get_child: (attributes org.gtk.Method.get_property=child)
+ * adw_toast_overlay_get_child:
  * @self: a toast overlay
  *
  * Gets the child widget of @self.
@@ -577,7 +601,7 @@ adw_toast_overlay_get_child (AdwToastOverlay *self)
 }
 
 /**
- * adw_toast_overlay_set_child: (attributes org.gtk.Method.set_property=child)
+ * adw_toast_overlay_set_child:
  * @self: a toast overlay
  * @child: (nullable): the child widget
  *
@@ -590,6 +614,9 @@ adw_toast_overlay_set_child (AdwToastOverlay *self,
   g_return_if_fail (ADW_IS_TOAST_OVERLAY (self));
   g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
 
+  if (child)
+    g_return_if_fail (gtk_widget_get_parent (child) == NULL);
+
   if (self->child == child)
     return;
 
